/*
 *  SynDetector
 *
 *  MSC in Telecommunications
 *  Denial of Service Study
 *	Queens University Belfast
 *
 *  Author   : Matthew Hutchinson
 *  Contact  : m.hutchinson@ijug.org
 *  Web      : http://hutchinson.ijug.org
 *
 *  Created  : 01-Aug-03
 *  Filename : ClientSocket.cpp
 *             Custom Socket extends CSniffSocket for TCP SYN packet monitoring
 *
 *  Library     : TCPIP_LIBv4 by Komodia Inc.  <open source>
 *	Download    : http://www.komodia.com/tcpip_lib4.zip
 *  API Support : http://www.komodia.com/help/default.htm 
 *  Copyright   : Copyright (c) 2000-2003 Barak Weichselbaum <barak@komodia.com>
 *                All rights reserved.
 *
 *  NOTICE      : This is a Proof Of Concept program for educational purposes only.
 *
 */

// for precompiled header directive
#include "stdafx.h"
// custom header files
#include "SynDetector.h"
#include "ClientSocket.h"
// TCPIP_LIBv4 library
#include "PingSocket.h"
#include "TCPSocket.h"

// maximum number of SYN packets from an unreachable IP address before warning is raised
#define MAXNUMSYNS 25


// global ping socket reference for message loop
CPingSocket* gpingSocket = NULL;
// counter to track number of SYN flood packets
static unsigned int synFloodCount;


// Class constructor definition
CClientSocket::CClientSocket(CListBox* pList, CEdit* fCount) 
: CSniffSocket(), m_pList(pList),  m_fCount(fCount)
{
}
CClientSocket::~CClientSocket()
{
}


/*
 *  BOOL ShouldStop()
 *  process for the message loop checks if PING is finished
 *
*/
BOOL ShouldStop()
{
	return gpingSocket->IsPingDone();
}


/*
 *  BOOL Bind(const std::string& unsigned short)
 *  Binds this socket to an IP address and port number
 *
*/
BOOL CClientSocket::Bind(const std::string& rSourceAddress, unsigned short usPort)
{
	// try to bind
	if (!CSpoofSocket::Bind(rSourceAddress,	usPort))
		return FALSE;
	else
	{
		// save source address
		m_sAddress=rSourceAddress;
		// reset the flood count for this socket binding
		synFloodCount = 0;
		return TRUE;
	}
}


/*
 *  BOOL OnSocketReceive(int)
 *  Method executed when any packet is recieved on the socket
 *
*/
BOOL CClientSocket::OnSocketReceive(int iErrorCode)
{
	// buffer to store all packet data
	char cBuffer[2000];

	// store packet data and check for errors
	int iReceive;
	iReceive = Receive(cBuffer, sizeof(cBuffer));
	if (iReceive == SOCKET_ERROR)
		return FALSE;

	// extract the IP header to an IpHeader object
	IpHeader aHeader;
	memcpy(&aHeader, cBuffer, IpHeaderLength);

	// get the packet source and destination IP
	std::string dAddress;
	std::string sAddress;
	dAddress = CSpoofSocket::LongToStdString(aHeader.ulDestinationAddress);
	sAddress = CSpoofSocket::LongToString(aHeader.ulSourceAddress);
	
	// if packet is directed at this machine anaylse it
	if(dAddress == m_sAddress)
	{
		// find the size of the IP header (may have options)
		unsigned long ulIPHeaderSize;
		ulIPHeaderSize = (aHeader.ucHeaderLength_Version & 0x0f) << 2;

		// check for a TCP packet
		if (aHeader.ucProtocol==IPPROTO_TCP) {
		
			// analyse TCP header for SYN packet and obtain destination Port
			int destPort;
			destPort = AnalyzeTCP(cBuffer+ulIPHeaderSize);

			if(destPort != -1)
			{		
				// format info message for SYN packet
				CString strInfo;
				char dataString[10];
				strInfo = "[SYN] packet -> PORT ";
				itoa(destPort, dataString, 10);
				strInfo += dataString;
				strInfo += " -> SOURCE : ";
				strInfo += sAddress.begin();
				// show info message
				m_pList->AddString(strInfo);

				// try to PING the source address to check it is ALIVE
				CPingSocket* pingSocket;
				pingSocket=new CPingSocket;
				// assign global ping socket for message loop
				gpingSocket = pingSocket;

				// create the ping Socket
				if (!pingSocket->Create())
					return FALSE;
				// ping the source address
				if (!pingSocket->Ping(sAddress))
				{
					// report unreachable SYN packet and add to flood counter
					m_pList->AddString("Ping Check : FAILED - bad host IP detected ...");
					synFloodCount++;					
				}
				else
				{
					// run a message loop while waiting for ping reply
					CAsyncSocket::SimpleMessageMap(ShouldStop, TRUE, 1);

					// check ping is finshed
					if (pingSocket->IsPingDone())
					{
						// did the ping request timeout ?
						if (pingSocket->IsTimeout())
						{
							// report unreachable SYN packet (timed out) and add to flood counter
							m_pList->AddString("Ping Check : FAILED - unreachable host timed out ...");
							synFloodCount++;
						}
						else
						{
							// if the ping checks out OK, show ICMP information
							strInfo = "Ping Check : OK ";

							strInfo += " : bytes=";
							itoa(pingSocket->GetDataSize(), dataString, 10);
							strInfo += dataString;
							strInfo += " time=";
							itoa(pingSocket->GetTimeElapsed(), dataString, 10);
							strInfo += dataString;
							strInfo += "ms TTL=";
							itoa(pingSocket->GetTTL(), dataString, 10);
							strInfo += dataString;

							// show info on ping reply
							m_pList->AddString(strInfo);
						}
					}
				}
			
				// update new flood counter value in main dialog
				itoa(synFloodCount, dataString, 10);
				m_fCount->SetWindowText(dataString);

				// check number of SYN packets has not exceeded limit
				if(synFloodCount == MAXNUMSYNS)
				{
					// format warning message
					strInfo = "WARNING !!!\nPossible SYN Flood in progress ...\n\n";
					strInfo += CSpoofSocket::LongToString(aHeader.ulDestinationAddress);
					strInfo += " has been targetted\n";
					itoa(synFloodCount, dataString, 10);
					strInfo += dataString;
					strInfo += " [SYN] packets detected on port ";	
					itoa(destPort, dataString, 10);
					strInfo += dataString;
					strInfo += "\n\nRemain calm and disable this port ...";
					// show warning message
					AfxMessageBox(strInfo, MB_ICONWARNING);
				}
				// newline after examining each packet
				m_pList->AddString("");
			}	
		}
	}

	return TRUE;
}


/*
 *  int AnalyzeTCP(char *)
 *  Examines TCP header data and checks for a SYN packet
 *  If found the destination Port is returned, or -1 for no match
 *
*/
int CClientSocket::AnalyzeTCP(char *cTCPBuffer)
{
	// read the TCP header
	TCPHeader tcpHeader;
	memcpy(&tcpHeader, cTCPBuffer, TCPHeaderLength);

	// get flags in HEXadecimal
	char cTCP_flags[10];
	itoa(tcpHeader.ucFlags, cTCP_flags, 16);
	
	// check for a SYN packet (0x0010 = 2) and return destination Port
	if(tcpHeader.ucFlags == 2)
		return htons(tcpHeader.usDestinationPort);
	
	// if no SYN packet found then return -1
	return -1;
}


/*
 *  ResetFloodCount()
 *  Reset the flood counter and update main dialog
 *
*/
void CClientSocket::ResetFloodCount()
{
	// reset flood count to 0 and update dialog display
	synFloodCount = 0;
	char data[10];
	itoa(synFloodCount, data, 10);
	m_fCount->SetWindowText(data);
}